"use strict";
/***********************************************************************
 * YoLink hub device support
 *
 * Homebridge does not have a hub-like device. But we register it and
 * log data about it.
 *
 * Copyright (c) 2022 David Kerr
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.mqttHubDevice = exports.initHubDevice = void 0;
/***********************************************************************
 * initHubDevice
 *
 */
async function initHubDevice() {
    // Homebridge does not have equivalent to a Hub device, so we essentially
    // just dummy out the functions for logging purposes.
    // Don't attempt to get data any more frequently than once an hour.
    this.config.refreshAfter = Math.max(3600, this.config.refreshAfter);
    this.refreshDataTimer(handleGet.bind(this));
}
exports.initHubDevice = initHubDevice;
/***********************************************************************
 * handleGet
 *
 */
async function handleGet() {
    const platform = this.platform;
    // serialize access to device data.
    const releaseSemaphore = await this.deviceSemaphore.acquire();
    try {
        const device = this.accessory.context.device;
        if (await this.checkDeviceState(platform, device)) {
            this.logDeviceState(`WiFi: ${device.data.wifi.enable}, Ethernet: ${device.data.eth.enable}`);
        }
        else {
            platform.log.error(`Device offline or other error for ${this.deviceMsgName}`);
        }
    }
    catch (e) {
        const msg = (e instanceof Error) ? e.stack : e;
        platform.log.error('Error in HubDevice handleGet' + platform.reportError + msg);
    }
    finally {
        await releaseSemaphore();
    }
    return (false);
}
/***********************************************************************
 * mqttHubDevice
 * According to YoLink documentation there is no callback for Hubs. But
 * setup a function for it just-in-case that is wrong.  Log data received.
 */
async function mqttHubDevice(message) {
    const platform = this.platform;
    // serialize access to device data.
    const releaseSemaphore = await this.deviceSemaphore.acquire();
    try {
        const device = this.accessory.context.device;
        // Merge received data into existing data object
        Object.assign(device.data, message.data);
        this.logDeviceState(`WiFi: ${device.data.wifi.enable}, Ethernet: ${device.data.eth.enable} (MQTT: ${message.event})`);
    }
    catch (e) {
        const msg = (e instanceof Error) ? e.stack : e;
        platform.log.error('Error in mqttHubDevice' + platform.reportError + msg);
    }
    finally {
        await releaseSemaphore();
    }
}
exports.mqttHubDevice = mqttHubDevice;
//# sourceMappingURL=hubDevice.js.map